/*
  Doc_Arduino - german Arduino Forum
  Programmed and tested with: Arduino IDE v1.8.13
  19.03.2021
  License: GNU GPLv3
*/

// Pin D13 wird mit D2 verbunden und an Pin 21 kommt der Schleiferkontakt eines Potis. //

#include <ForwardCompatibility.h>
#include <megaAVR0TCBn.h>
#include <NanoEveryPin.h>
#include <avr/interrupt.h>
#include <util/atomic.h>

TCB          <0> timerTCB0;     // TCB Timer Nummer 0
InputPin     <2> messPinTCB0;   // Eingangspin für TCB0 für Timeout Messung, (pinPA0)
OutputPin   <13> onboardLed;    // erzeugt das Timeout Signal für TCB0, (pinPE2)
AnalogInPin <21> analogIn;      // analoger Messeingang für ADC0, (pinPD5)

struct DatenADC0
{
  volatile unsigned int adcVal;
  bool finish;
};
DatenADC0 adc0;

void setup()
{
  Serial.begin(250000);
  Serial.println(F("\nStart #### #### #### ####"));
  
  messPinTCB0.init();
  onboardLed.init();
  analogIn.init();
  initTCA0();        
  initTCB0();
  initADC0();
  initEventSystem();

  onboardLed.setOn();  // dient als Event Trigger Start für TCB0
}

void loop()
{
  if (adc0.finish)
  {
    uint16_t var {0};
    ATOMIC_BLOCK (ATOMIC_RESTORESTATE)
    {
      var = adc0.adcVal >> 4; // geteilt durch 16fach Akkumulator Sampling
    }
    Serial.print("ADC0 "); Serial.print(var); Serial.print(" Digits");
    Serial.println();
    adc0.finish = false;
    onboardLed.setOn();
  }
}

void initTCA0(void)
{
  // IDE default: TCA: 0b1011 > DIV64 & Enable > 250kHz > 4µs // 
  TCA0.SINGLE.CTRLA = TCA0.SINGLE.CTRLA | TCA_SINGLE_CLKSEL_DIV1024_gc;
}

void initADC0(void)
{
  /* 
     IDE default:
     ADC0.CTRLA   // 0b0000'0001 -->> enable
     ADC0.CTRLC   // 0b0001'0110 -->> DIV128 und Vref = Vdd
  */
  ADC0.CTRLA = 0;                       // ADC stoppen  
  ADC0.MUXPOS = ADC_MUXPOS_AIN5_gc;     // select ADC0 channel 5, (pinPD5)
  ADC0.INTCTRL = ADC_RESRDY_bm;         // enable interrupts
  ADC0.EVCTRL = ADC_STARTEI_bm;         // enable event triggered conversion
  ADC0.CTRLB = ADC_SAMPNUM_ACC16_gc;    // 16fach Oversampling mit internen Akkumulator
  ADC0.CTRLA = ADC_ENABLE_bm;           // ADC einschalten  
}

ISR(ADC0_RESRDY_vect)
{
  onboardLed.setOff();
  adc0.adcVal = ADC0.RES;
  adc0.finish = true;
  ADC0.INTFLAGS = ADC_RESRDY_bm;        // löscht das 'Result ready interrupt' Flag
}

void initTCB0(void)
{
  timerTCB0.reset();
  timerTCB0.setCLKTCA();
  timerTCB0.setModeTIMEOUT();
  timerTCB0.setCompare(15625);     // 62,5ns * 1024 = 64µs -->> 64µs * 15625 = 1s
  timerTCB0.enableEventInput();    // Event-Trigger aktivieren, damit Timeout Interrupt ausgelöst wird
  timerTCB0.enable();
}

void initEventSystem(void)
{
  EVSYS.CHANNEL1 = EVSYS_GENERATOR_PORT0_PIN0_gc;   // verbinde Generator 'Pin D2' (PA0) mit Kanal 1
  EVSYS.USERTCB0 = EVSYS_CHANNEL_CHANNEL1_gc;       // verbinde User 'TCB0' mit Kanal 1
  
  EVSYS.CHANNEL0 = EVSYS_GENERATOR_TCB0_CAPT_gc;    // verbinde Generator 'TCB0 Timeout Event' mit Kanal 0
  EVSYS.USERADC0 = EVSYS_CHANNEL_CHANNEL0_gc;       // verbinde User 'ADC0' mit Kanal 0
}
